const_iv(SUCCESS)
#ifdef CL_DEVICE_NOT_FOUND
const_iv(DEVICE_NOT_FOUND)
#endif
#ifdef CL_DEVICE_NOT_AVAILABLE
const_iv(DEVICE_NOT_AVAILABLE)
#endif
#ifdef CL_COMPILER_NOT_AVAILABLE
const_iv(COMPILER_NOT_AVAILABLE)
#endif
#ifdef CL_MEM_OBJECT_ALLOCATION_FAILURE
const_iv(MEM_OBJECT_ALLOCATION_FAILURE)
#endif
#ifdef CL_OUT_OF_RESOURCES
const_iv(OUT_OF_RESOURCES)
#endif
#ifdef CL_OUT_OF_HOST_MEMORY
const_iv(OUT_OF_HOST_MEMORY)
#endif
#ifdef CL_PROFILING_INFO_NOT_AVAILABLE
const_iv(PROFILING_INFO_NOT_AVAILABLE)
#endif
#ifdef CL_MEM_COPY_OVERLAP
const_iv(MEM_COPY_OVERLAP)
#endif
#ifdef CL_IMAGE_FORMAT_MISMATCH
const_iv(IMAGE_FORMAT_MISMATCH)
#endif
#ifdef CL_IMAGE_FORMAT_NOT_SUPPORTED
const_iv(IMAGE_FORMAT_NOT_SUPPORTED)
#endif
#ifdef CL_BUILD_PROGRAM_FAILURE
const_iv(BUILD_PROGRAM_FAILURE)
#endif
#ifdef CL_MAP_FAILURE
const_iv(MAP_FAILURE)
#endif
#ifdef CL_MISALIGNED_SUB_BUFFER_OFFSET
const_iv(MISALIGNED_SUB_BUFFER_OFFSET)
#endif
#ifdef CL_EXEC_STATUS_ERROR_FOR_EVENTS_IN_WAIT_LIST
const_iv(EXEC_STATUS_ERROR_FOR_EVENTS_IN_WAIT_LIST)
#endif
#ifdef CL_INVALID_VALUE
const_iv(INVALID_VALUE)
#endif
#ifdef CL_INVALID_DEVICE_TYPE
const_iv(INVALID_DEVICE_TYPE)
#endif
#ifdef CL_INVALID_PLATFORM
const_iv(INVALID_PLATFORM)
#endif
#ifdef CL_INVALID_DEVICE
const_iv(INVALID_DEVICE)
#endif
#ifdef CL_INVALID_CONTEXT
const_iv(INVALID_CONTEXT)
#endif
#ifdef CL_INVALID_QUEUE_PROPERTIES
const_iv(INVALID_QUEUE_PROPERTIES)
#endif
#ifdef CL_INVALID_COMMAND_QUEUE
const_iv(INVALID_COMMAND_QUEUE)
#endif
#ifdef CL_INVALID_HOST_PTR
const_iv(INVALID_HOST_PTR)
#endif
#ifdef CL_INVALID_MEM_OBJECT
const_iv(INVALID_MEM_OBJECT)
#endif
#ifdef CL_INVALID_IMAGE_FORMAT_DESCRIPTOR
const_iv(INVALID_IMAGE_FORMAT_DESCRIPTOR)
#endif
#ifdef CL_INVALID_IMAGE_SIZE
const_iv(INVALID_IMAGE_SIZE)
#endif
#ifdef CL_INVALID_SAMPLER
const_iv(INVALID_SAMPLER)
#endif
#ifdef CL_INVALID_BINARY
const_iv(INVALID_BINARY)
#endif
#ifdef CL_INVALID_BUILD_OPTIONS
const_iv(INVALID_BUILD_OPTIONS)
#endif
#ifdef CL_INVALID_PROGRAM
const_iv(INVALID_PROGRAM)
#endif
#ifdef CL_INVALID_PROGRAM_EXECUTABLE
const_iv(INVALID_PROGRAM_EXECUTABLE)
#endif
#ifdef CL_INVALID_KERNEL_NAME
const_iv(INVALID_KERNEL_NAME)
#endif
#ifdef CL_INVALID_KERNEL_DEFINITION
const_iv(INVALID_KERNEL_DEFINITION)
#endif
#ifdef CL_INVALID_KERNEL
const_iv(INVALID_KERNEL)
#endif
#ifdef CL_INVALID_ARG_INDEX
const_iv(INVALID_ARG_INDEX)
#endif
#ifdef CL_INVALID_ARG_VALUE
const_iv(INVALID_ARG_VALUE)
#endif
#ifdef CL_INVALID_ARG_SIZE
const_iv(INVALID_ARG_SIZE)
#endif
#ifdef CL_INVALID_KERNEL_ARGS
const_iv(INVALID_KERNEL_ARGS)
#endif
#ifdef CL_INVALID_WORK_DIMENSION
const_iv(INVALID_WORK_DIMENSION)
#endif
#ifdef CL_INVALID_WORK_GROUP_SIZE
const_iv(INVALID_WORK_GROUP_SIZE)
#endif
#ifdef CL_INVALID_WORK_ITEM_SIZE
const_iv(INVALID_WORK_ITEM_SIZE)
#endif
#ifdef CL_INVALID_GLOBAL_OFFSET
const_iv(INVALID_GLOBAL_OFFSET)
#endif
#ifdef CL_INVALID_EVENT_WAIT_LIST
const_iv(INVALID_EVENT_WAIT_LIST)
#endif
#ifdef CL_INVALID_EVENT
const_iv(INVALID_EVENT)
#endif
#ifdef CL_INVALID_OPERATION
const_iv(INVALID_OPERATION)
#endif
#ifdef CL_INVALID_GL_OBJECT
const_iv(INVALID_GL_OBJECT)
#endif
#ifdef CL_INVALID_BUFFER_SIZE
const_iv(INVALID_BUFFER_SIZE)
#endif
#ifdef CL_INVALID_MIP_LEVEL
const_iv(INVALID_MIP_LEVEL)
#endif
#ifdef CL_INVALID_GLOBAL_WORK_SIZE
const_iv(INVALID_GLOBAL_WORK_SIZE)
#endif
#ifdef CL_INVALID_PROPERTY
const_iv(INVALID_PROPERTY)
#endif
#ifdef CL_PLATFORM_NOT_FOUND_KHR
const_iv(PLATFORM_NOT_FOUND_KHR)
#endif
#ifdef CL_INVALID_GL_SHAREGROUP_REFERENCE_KHR
const_iv(INVALID_GL_SHAREGROUP_REFERENCE_KHR)
#endif
#ifdef CL_INVALID_D3D10_DEVICE_KHR
const_iv(INVALID_D3D10_DEVICE_KHR)
#endif
#ifdef CL_INVALID_D3D10_RESOURCE_KHR
const_iv(INVALID_D3D10_RESOURCE_KHR)
#endif
#ifdef CL_D3D10_RESOURCE_ALREADY_ACQUIRED_KHR
const_iv(D3D10_RESOURCE_ALREADY_ACQUIRED_KHR)
#endif
#ifdef CL_D3D10_RESOURCE_NOT_ACQUIRED_KHR
const_iv(D3D10_RESOURCE_NOT_ACQUIRED_KHR)
#endif
